@@ -1,4 +1,4 @@
-/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2023 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@
 /*** Query module API. ***/
 
 /*! Current module ABI version. */
-#define KNOTD_MOD_ABI_VERSION	300
+#define KNOTD_MOD_ABI_VERSION	500
 /*! Module configuration name prefix. */
 #define KNOTD_MOD_NAME_PREFIX	"mod-"
 
@@ -154,7 +154,7 @@ int knotd_conf_check_ref(knotd_conf_check_args_t *args);
  *
  * \return Pointer to optional module context.
  */
-void *knotd_mod_ctx(knotd_mod_t *mod);
+void *knotd_mod_ctx(const knotd_mod_t *mod);
 
 /*!
  * Sets optional module context.
@@ -171,7 +171,7 @@ void knotd_mod_ctx_set(knotd_mod_t *mod, void *ctx);
  *
  * \return Zone name.
  */
-const knot_dname_t *knotd_mod_zone(knotd_mod_t *mod);
+const knot_dname_t *knotd_mod_zone(const knotd_mod_t *mod);
 
 /*!
  * Emits a module specific log message.
@@ -388,22 +388,31 @@ typedef enum {
 	KNOTD_QUERY_TYPE_UPDATE,  /*!< Dynamic update. */
 } knotd_query_type_t;
 
+/*! Supported transport protocols. */
+typedef enum {
+	KNOTD_QUERY_PROTO_UDP  = KNOT_PROBE_PROTO_UDP,  /*!< Pure UDP. */
+	KNOTD_QUERY_PROTO_TCP  = KNOT_PROBE_PROTO_TCP,  /*!< Pure TCP. */
+	KNOTD_QUERY_PROTO_QUIC = KNOT_PROBE_PROTO_QUIC, /*!< QUIC/UDP. */
+} knotd_query_proto_t;
+
 /*! Query processing specific flags. */
 typedef enum {
-	KNOTD_QUERY_FLAG_NO_AXFR    = 1 << 0, /*!< Don't process AXFR. */
-	KNOTD_QUERY_FLAG_NO_IXFR    = 1 << 1, /*!< Don't process IXFR. */
-	KNOTD_QUERY_FLAG_LIMIT_SIZE = 1 << 2, /*!< Apply UDP size limit. */
-	KNOTD_QUERY_FLAG_COOKIE     = 1 << 3, /*!< Valid DNS Cookie indication. */
+	KNOTD_QUERY_FLAG_COOKIE  = 1 << 0, /*!< Valid DNS Cookie indication. */
 } knotd_query_flag_t;
 
 /*! Query processing data context parameters. */
 typedef struct {
-	knotd_query_flag_t flags;              /*!< Current query flgas. */
+	knotd_query_proto_t proto;             /*!< Transport protocol used. */
+	knotd_query_flag_t flags;              /*!< Current query flags. */
 	const struct sockaddr_storage *remote; /*!< Current remote address. */
+	const struct sockaddr_storage *local;  /*!< Current local address. */
 	int socket;                            /*!< Current network socket. */
 	unsigned thread_id;                    /*!< Current thread id. */
 	void *server;                          /*!< Server object private item. */
 	const struct knot_xdp_msg *xdp_msg;    /*!< Possible XDP message context. */
+	struct knot_quic_conn *quic_conn;      /*!< QUIC connection context. */
+	int64_t quic_stream;                   /*!< QUIC stream ID inside quic_conn. */
+	uint32_t measured_rtt;                 /*!< Measured RTT in usecs: QUIC or TCP-XDP. */
 } knotd_qdata_params_t;
 
 /*! Query processing data context. */
@@ -430,12 +439,13 @@ typedef struct {
  * Gets the local (destination) address of the query.
  *
  * \param[in] qdata  Query data.
- * \param[out] buff  Auxiliary buffer (not used for XDP).
  *
  * \return Local address or NULL if error.
  */
-const struct sockaddr_storage *knotd_qdata_local_addr(knotd_qdata_t *qdata,
-                                                      struct sockaddr_storage *buff);
+inline static const struct sockaddr_storage *knotd_qdata_local_addr(const knotd_qdata_t *qdata)
+{
+	return qdata->params->local;
+}
 
 /*!
  * Gets the remote (source) address of the query.
@@ -444,7 +454,19 @@ const struct sockaddr_storage *knotd_qdata_local_addr(knotd_qdata_t *qdata,
  *
  * \return Remote address or NULL if error.
  */
-const struct sockaddr_storage *knotd_qdata_remote_addr(knotd_qdata_t *qdata);
+inline static const struct sockaddr_storage *knotd_qdata_remote_addr(const knotd_qdata_t *qdata)
+{
+	return qdata->params->remote;
+}
+
+/*!
+ * Gets the measured TCP round-trip-time.
+ *
+ * \param[in] qdata  Query data.
+ *
+ * \return RTT in microseconds or 0 if error or not available.
+ */
+uint32_t knotd_qdata_rtt(knotd_qdata_t *qdata);
 
 /*!
  * Gets the current zone name.
@@ -453,7 +475,7 @@ const struct sockaddr_storage *knotd_qdata_remote_addr(knotd_qdata_t *qdata);
  *
  * \return Zone name.
  */
-const knot_dname_t *knotd_qdata_zone_name(knotd_qdata_t *qdata);
+const knot_dname_t *knotd_qdata_zone_name(const knotd_qdata_t *qdata);
 
 /*!
  * Gets the current zone apex rrset of the given type.
@@ -463,16 +485,7 @@ const knot_dname_t *knotd_qdata_zone_name(knotd_qdata_t *qdata);
  *
  * \return A copy of the zone apex rrset.
  */
-knot_rrset_t knotd_qdata_zone_apex_rrset(knotd_qdata_t *qdata, uint16_t type);
-
-/*!
- * Gets the original QNAME of the query, before downcasing.
- *
- * \param[in] qdata  Query data.
- *
- * \return Original QNAME or NULL if error.
- */
-const knot_dname_t *knotd_qdata_orig_qname(knotd_qdata_t *qdata);
+knot_rrset_t knotd_qdata_zone_apex_rrset(const knotd_qdata_t *qdata, uint16_t type);
 
 /*! General query processing states. */
 typedef enum {
